<?php
/* vim:set expandtab tabstop=4 shiftwidth=4 softtabstop=4 foldmethod=marker syntax=php: */
// {{{ firephp
/**
 * This replaces the behavior of {@link com__googlecode__firephp__FirePHP_class}
 * with something more drop-in (output buffer) friendly.
 *
 * @author terry chay <tychay@php.net>
 * @author Christoph Dorn <christop@christophdorn.com> (original code)
 * @license Mozilla Public License
 */
class firephp
{
    var $firebugContent = '';
    var $inspectorTarget = '';
    var $_requestId = '';
    var $_primaryContentType = 'text/html';
    var $_contentStarted = false;
    var $_multipartRequested = false;
    // ACCESSORS
    // {{{ - setRequestId($requestId)
    function setRequestID($requestId)
    {
        if($this->_contentStarted) {
            trigger_error('Content has already started. You set the request ID before calling startContent()!');
            return false;
        }
        $this->_requestId = $requestId;
        $this->setHeaderVariable('RequestID',$this->_requestId);
        return true;
    }
    // }}}
    // {{{ - setUniqueId()
    function generateUniqId()
    {
        $this->setRequestID(md5(uniqid(rand(), true)));
    }
    // }}}
    // {{{ - setPrimaryContentType($contentType)
    function setPrimaryContentType($contentType)
    {
        if($this->_contentStarted) {
            trigger_error('Content has already started. You must set the content type before calling startContent()!');
            return false;
        }
        $this->_primaryContentType = $contentType;
        return true;
    }
    // }}}
    // PUBLIC METHODS
    // {{{ - setHeaderVariable()
    /**
     * Send global headers that are read by FirePHP segment.
     */
    function setHeaderVariable($key,$value)
    {
        if ($this->_contentStarted) {
            trigger_error('Content has already started. You must set header variables before calling startContent()!');
            return false;
        }
        if(headers_sent($file,$line)) {
            trigger_error('Headers already sent in file['.$file.'] line['.$line.']!');
            return false;
        } else { 
            header(sprintf('PINF-org.firephp-%s: %s', $key, $value));
            return true;
        }
    }  
    // }}}
    // {{{ - doesBrowserAccept()
    /**
     * Search to see if FirePHP is supported.
     *
     * Add to this for security.
     * @return boolean whether or not to enable FirePHP channel.
     */
    function doesBrowserAccept() {
        // make sure accept content type "text/firephp" sent by client {{{
        if(!preg_match_all("/(^|,|\s)(text\/firephp)($|,|\s)/si",$_SERVER['HTTP_ACCEPT'],$m)) {
            return false;
        }
        // }}}
        // Add more checks here...
        return true;
    }
    // }}}
    // {{{ - enableMultipartData()
    /**
     * Enable the multipart/firephp server response by setting the header
     * and keeping a flag that we can collect the data.
     *
     * There is no need to collect the data and slow down PHP if we cannot
     * even send it.
     */ 
    function enableMultipartData()
    {
        if($this->_contentStarted) {
            trigger_error('Content has already started. You must enable multipart data before calling startContent()!');
            return false;
        }
        if(headers_sent($file,$line)) {
            trigger_error('Headers already sent in file['.$file.'] line['.$line.']!');
            return false;
        } else { 
            if(!$this->_requestId) {
                trigger_error('The RequestID must be set first before enabeling the multipart response!');
                return false;
            }
            $this->_multipartRequested = true;
            return true;
        }
    }
    // }}}
    // {{{ - getContent($mainPart[,$contentType])
    /**
     * Generate the missing headers and the payload.
     *
     * @return the HTTP body
     * @todo no support for inspector target
     */
    function getContent($mainPart, $contentType=false)
    {
        if ($contentType) {
            if (!$this->setPrimaryContentType($contentType)) { return $mainPart; }
        }
        if(headers_sent($file,$line)) {
            trigger_error('Headers already sent in file['.$file.'] line['.$line.']!');
            return $mainPart;
        }
        if(!$this->_multipartRequested) {
            header(sprintf('Content-type: %s',$this->_primaryContentType));
            return $mainPart;
        }
        // Set the multipart mixed header
        header('Content-type: multipart/mixed; boundary="'.$this->_requestId.'"');
        // Ensure that the request is never cached by the browser {{{
        header('Last-Modified: '.gmdate('r', time()));
        header('Expires: '.gmdate('r', time()-86400));
        header('Pragma: no-cache');
        header('Cache-Control: no-cache, no-store, must-revalidate, max_age=0');
        header('Cache-Control: post-check=0, pre-check=0'); 
        $this->_contentStarted=true;
        // }}}
        // main part {{{
        $return = sprintf("--%s\nContent-type: %s\n\n%s", $this->_requestId, $this->_primaryContentType, $mainPart);
        // }}}
        // firephp part {{{
        $return .= sprintf("\n--%s\nContent-type: text/firephp\n\n",$this->_requestId);
        $return .= sprintf('<firephp><request id="%s" anchor="%s"><data type="html"><![CDATA[%s]]></data></request></firephp>', $this->_requestId, $this->inspectorTarget, trim($this->firebugContent));
        // }}}
        $return .= sprintf("\n--%s--\n",$this->_requestId);
        return $return;
    }
    // }}}
    // TIMERS
    // {{{ - stampTime($group[,$marker])
    function stampTime($group,$marker='start')
    {
        $this->timeMarkers[$group][$marker] = firephp::getAccurateTime();
        return true;
    }
    // }}}
    // {{{ - stopTimer($group)
    function stopTimer($group)
    {
        $this->stampTime($group,'stop');
        return $this->getTimeSpan($group,'start','stop');
    }
    // }}}
    // {{{ - getTimeSpan($group,$startMarker,$endMarker)
    function getTimeSpan($group,$startMarker,$endMarker)
    {
        return $this->timeMarkers[$group][$endMarker] - $this->timeMarkers[$group][$startMarker];
    }
    // }}}
    // {{{ + getAccurateTime()
    /**
     * I like this more than {@link microtime()} in {@link FirePHP.class.php}.
     * But you can do whatever.
     */
    function getAccurateTime() {
        $tod = gettimeofday();
        return $tod['sec'] + ($tod['usec'] / 1000000);
    }
    // }}}
}
// }}}
?>
